/*
 * This example sketch shows how you can manage the nRF5 pin mapping as part of your code.
 * You can use the method for any nRF51822 or nRF52832 board or module.
 * 
 * Most components, like UART, SPI, Wire Bus, of the nRF5 series chips don't
 * have a fixed pin mapping. There are some pins with restrictions like analog
 * inputs, NFC or pins near the radio module. Please refer the latest
 * documentation about pin restrictions at http://infocenter.nordicsemi.com 
 * 
 * To use the custom pin mapping you have to do following steps:
 * 
 * 1. Install "arduino-nrf5" like described at
 *    https://github.com/sandeepmistry/arduino-nRF5/
 * 2. Install the "My Sensors nRF5 Boards" with the board manager like
 *    explained at https://github.com/mysensors/ArduinoBoards
 * 3. Copy the files "MyBoardNRF5.cpp" and "MyBoardNRF5.h" from
 *    "MyBoardNRF5" example into your sketch.
 * 4. Modify pin mappings in "MyBoardNRF5.cpp" and "MyBoardNRF5.h" to fit
 *    your requirements.
 * 5. Select "MyBoardNRF5 nrf52832" or "MyBoardNRF5 nrf52822" as your board.
 *    Choose the correct parameters and programmer in the Tools menu.
 */

// MySensors /////////////////////
#define MY_DEBUG
#define MY_RADIO_NRF5_ESB
#define MY_NODE_ID 216
//#define MY_PASSIVE_NODE

#include <MySensors.h>
//#include <nrf.h>


#define SKETCH_NAME "Jung Node TEST NR2"
#define SKETCH_VERSION "v0.4"

#define CHILD_ID_HUM 0
#define CHILD_ID_TEMP 1
#define CHILD_ID_TRIP 2 
#define CHILD_ID_VOLT 28


MyMessage msgHum(CHILD_ID_HUM, V_HUM);
MyMessage msgTemp(CHILD_ID_TEMP, V_TEMP);
MyMessage msgTrip(CHILD_ID_TRIP, V_TRIPPED);
MyMessage msgVoltage(CHILD_ID_VOLT,V_VOLTAGE);

// Pins ///////////////////////
#define LED_PIR_Movement     29  // LED underneath PIR Cap
#define LED_GeneralPurpose    6  // on board LED

#define PIR_Pin               2  // Data line from PIR


// Timing etc //////////////////
#define SHORT_WAIT 50
#define LongSleep 30000    //(600.000 = 10 minutes) (-200 to counter shifting)
long Interval_StartSleep;   // Sleeping interval check
long Interval_Sleeptime;    // Sleeping interval check


// SI7021  /////////////////////
#include "SparkFun_Si7021_Breakout_Library.h"
#include <Wire.h>
#include <SPI.h>

static const uint8_t FORCE_UPDATE_N_READS = 20; //Send update every 10 turns

Weather sensor;
  float lastTemp;
  float temperature;
  float lastHum;
  float humidity;
  uint8_t nNoUpdatesTemp;
  uint8_t nNoUpdatesHum;
  bool metric = true;

int movementCheck; // used for checking movement,,,


void presentation()
{
  Serial.println("Presentation!");
  // Send the sketch version information to the gateway
  sendSketchInfo(SKETCH_NAME, SKETCH_VERSION);
  Serial.println("Presentation!");
  // Register all sensors to gw (they will be created as child devices)
  present(CHILD_ID_HUM, S_HUM);
  wait(SHORT_WAIT);
  present(CHILD_ID_TEMP, S_TEMP);
  wait(SHORT_WAIT);
  present(CHILD_ID_TRIP, S_MOTION);
  wait(SHORT_WAIT);
  present(CHILD_ID_VOLT,S_MULTIMETER);

}


void setup() {
  Serial.begin(115200);

  sensor.begin(); // SI7021
  
  pinMode(LED_PIR_Movement , OUTPUT);
  pinMode(LED_GeneralPurpose , OUTPUT);      
  pinMode(PIR_Pin, INPUT);   

  
  // On first startup, give the temp and humidity
  getWeather();      
  //send(msgTemp.set(temperature, 1));
  //send(msgHum.set(humidity, 1));
  i2c_off(); // Shut down the i2c
}

void loop() 
{


movement();
environment();
 

}// loop

void i2c_off() // Shut i2c down, powerconsumption rises up to 450uA instead of 11 ua  - https://github.com/sandeepmistry/arduino-nRF5/issues/291#issuecomment-407492282
{
  NRF_TWI1->ENABLE=TWI_ENABLE_ENABLE_Disabled << TWI_ENABLE_ENABLE_Pos;
*(volatile uint32_t *)0x40004FFC = 0;
*(volatile uint32_t *)0x40004FFC;
*(volatile uint32_t *)0x40004FFC = 1;

}


void movement()
{
  /*
  // First we check if we got movement before sleeping, if so, for instance when you got a bussy household, send movement to the controller.
  // The AM612 can be HIGH just before we start sleeping
  movementCheck = digitalRead(PIR_Pin); 
  if (movementCheck == 1)
  {  Serial.println("you got a bussy household");
     send(msgTrip.set(millis));// Sending something
  }*/
 Interval_StartSleep = millis(); // Register where we started sleeping
 
 sleep(digitalPinToInterrupt(PIR_Pin), CHANGE, LongSleep); // Here the waiting starts
 Interval_Sleeptime = millis()-Interval_StartSleep;
 //Serial.println("we are awake jeeh, party hard while i can!");   // DEBUG

 if (Interval_Sleeptime < LongSleep)
  {
  Serial.print("Captains Log, I woke up because I saw movement at: ");Serial.println(millis()); // DEBUG
  
  
  digitalWrite(LED_PIR_Movement, HIGH); wait(10);digitalWrite(LED_PIR_Movement, LOW);//Blinking because we can
  
  
  send(msgTrip.set(millis));// Sending something
  
  //Blinking because we can
  digitalWrite(LED_GeneralPurpose, HIGH); wait(10);digitalWrite(LED_GeneralPurpose, LOW); //Blinking because we can
  
  Serial.print("Captains Log, Reported the incident to the federation, I slept for : ");Serial.print(Interval_Sleeptime);   // DEBUG
  
  Interval_Sleeptime = LongSleep- Interval_Sleeptime;
  if (Interval_Sleeptime <= 0 || Interval_Sleeptime >= LongSleep) // for the slim chance millis() gets confused or something, check if the remainder of sleep is still viable.
   {
   Serial.print("             Captains Log, WHOOO, I GOT THE MATH WRONG! : ");Serial.print(Interval_Sleeptime);
   Interval_Sleeptime = 150000; // just a time 
       
   }
   Serial.print("             Captains Log, Everything is quite again, let sleep for another: ");Serial.println(Interval_Sleeptime); // DEBUG
   sleep(Interval_Sleeptime); // Sleep for the remainder of the 10 minutes
 
  }
 
}

void environment()
{
  sensor.begin(); // Because the i2c is shut down, call this one again.
  
 // Serial.println("Reading Temp on timed basis");
  
    // Wakeup for Temp data
      getWeather(); 
     // Serial.println("getWeather = Pass");
  if (isnan(temperature)) {
      Serial.println("Failed reading temperature from Si7021!");
      }
    else if (temperature != lastTemp || nNoUpdatesTemp == FORCE_UPDATE_N_READS) {
      // Only send temperature if it changed since the last measurement or if we didn't send an update for n times
      lastTemp = temperature;
      // Reset no updates counter
      nNoUpdatesTemp = 0;
      send(msgTemp.set(temperature, 1));
      } 
    else {
      // Increase no update counter if the temperature stayed the same
      nNoUpdatesTemp++;
      }


  if (isnan(humidity)) {

      }
    else if (humidity != lastHum || nNoUpdatesHum == FORCE_UPDATE_N_READS) {
      // Only send humidity if it changed since the last measurement or if we didn't send an update for n times
      lastHum = humidity;
      // Reset no updates counter
      nNoUpdatesHum = 0;
      send(msgHum.set(humidity, 1));
  
      } 
      else {
        nNoUpdatesHum++;
      }
 
i2c_off();
  
}


void getWeather()
{
  // Measure Relative Humidity from the HTU21D or Si7021
  humidity = sensor.getRH();

  // Measure Temperature from the HTU21D or Si7021
  temperature = sensor.getTemp();
  // Temperature is measured every time RH is requested.
  // It is faster, therefore, to read it from previous RH
  // measurement with getTemp() instead with readTemp()
}

